#import "SoundEffect.h"

 #import "BreakView.h"

@implementation	BreakView	(DrawingSound)	// --------------------
- (void)setBackgroundFile:(NSString *)fileName andRemember:(BOOL)remember;
 {
    [backGround release];
    if (fileName) {
		backGround = [[NSImage allocWithZone:[self zone]] initByReferencingFile:fileName];
		[backGround setScalesWhenResized:YES];
		[backGround setSize:gameSize];
		if (remember) {
			[[NSUserDefaults standardUserDefaults] setObject:fileName forKey:@"BackGround"];
		}
    } else {
		backGround = [[NSImage allocWithZone:[self zone]] initWithSize:gameSize];
		[backGround addRepresentation:[[[NSCustomImageRep alloc] initWithDrawSelector:@selector(drawDefaultBackground:) 
																						delegate:self] autorelease]];
		[backGround setScalesWhenResized:NO];
		if (remember) {
			[[NSUserDefaults standardUserDefaults] removeObjectForKey:@"BackGround"];
		}
    }
    [backGround setBackgroundColor:[NSColor whiteColor]];
    [self setNeedsDisplay:YES];
 }

 // The following two methods allow changing the background image from
 // menu items or buttons.

- (void)changeBackground:sender;		
 {
    NSOpenPanel *openPanel = [NSOpenPanel openPanel];
    if ([openPanel runModalForTypes:[NSImage imageFileTypes]]) {
	[self setBackgroundFile:[openPanel filename] andRemember:YES];
        [self setNeedsDisplay:YES];
    }
 }

- (void)revertBackground:sender;		
 {
    [self setBackgroundFile:NULL andRemember:YES];
    [self setNeedsDisplay:YES];
 }

 // getHighScore reads the previous high score from the user's defaults file.
 // If no such default is found, then the high score is set to zero.

 #define NUMYBOXES 10
 #define NUMXBOXES 6

 // This method draws the default background. The default background consists of
 // NUMXBOXES x NUMYBOXES raised boxes. Each box is drawn as four triangles to
 // provide a raised effect. Boxes near the top left corner are lighter in color
 // than the ones near the bottom right.

static void drawRectangularShadowUnder (NSRect rect, float offset)	
 {
    NSRect shadeRect = NSOffsetRect(rect, offset, -offset);
    [[[NSColor blackColor] colorWithAlphaComponent:0.333] set];
    NSRectFill(shadeRect);
 }

 // Draws a 3d box; if inset == YES, the box is inset

static void draw3DBox (NSRect rect, BOOL inset)						
 {
    static NSColor *colors[9] = {nil};
    NSRect rects[9];

    // Convenience macros...
    #define RX rect.origin.x
    #define RY rect.origin.y
    #define RW rect.size.width
    #define RH rect.size.height
    #define RMX (RX + RW - 1)
    #define RMY (RY + RH - 1)

    if (colors[0] == nil) {	// Fill these in first time through
        colors[0] = colors[1] = [[NSColor colorWithCalibratedWhite:0.666 alpha:1.0] retain];
        colors[2] = colors[3] = [[NSColor colorWithCalibratedWhite:1.0 alpha:1.0] retain];
        colors[4] = colors[5] = [[NSColor colorWithCalibratedWhite:0.0 alpha:1.0] retain];
        colors[6] = colors[7] = [[NSColor colorWithCalibratedWhite:0.5 alpha:1.0] retain];
        colors[8] = [[NSColor colorWithCalibratedWhite:0.875 alpha:1.0] retain];
    }
    rects[inset ? 6 : 0] = NSMakeRect(RX, RY + 1, 1, RH - 2);
    rects[inset ? 7 : 1] = NSMakeRect(RX + 1, RMY, RW - 2, 1);
    rects[inset ? 4 : 2] = NSMakeRect(RX + 1, RY + 1, 1, RH - 2);
    rects[inset ? 5 : 3] = NSMakeRect(RX + 1, RMY - 1, RW - 2, 1);
    rects[inset ? 2 : 4] = NSMakeRect(RMX, RY + 1, 1, RH - 2);
    rects[inset ? 3 : 5] = NSMakeRect(RX + 1, RY, RW - 2, 1);
    rects[inset ? 0 : 6] = NSMakeRect(RMX - 1, RY + 1, 1, RH - 3);
    rects[inset ? 1 : 7] = NSMakeRect(RX + 2, RY + 1, RW - 3, 1);
    rects[8] = NSMakeRect(RX + 2, RY + 2, RW - 4, RH - 4);
    NSRectFillListWithColors(rects, colors, 9);
 }

- (void)drawBall:imageRep;			
 {
    NSSize circleSize = NSMakeSize(BALLWIDTH - SHADOWOFFSET, BALLHEIGHT - SHADOWOFFSET);
    
    // Drawing the ball is tricky; if the view is resized, we want to draw an ellipse
    // (not circle), so we have to scale. When we scale, we want the distance 
    // between the shadow and the ball to remain the same, so we scale and draw 
    // the shadow and the ball separately...

    PSgsave();
    // First draw the shadow under the ball.
    PStranslate(SHADOWOFFSET, 0);
    PSscale ((ballSize.width - SHADOWOFFSET) / circleSize.width, (ballSize.height - SHADOWOFFSET) / circleSize.height);
    [[[NSColor blackColor] colorWithAlphaComponent:0.333] set];
    PSarc (RADIUS-SHADOWOFFSET, RADIUS-SHADOWOFFSET, RADIUS-SHADOWOFFSET, 0.0, 360.0);
    PSfill ();
    PSgrestore();

    PSgsave();
    
    // Then the ball.
    PStranslate(0, SHADOWOFFSET);
    PSscale ((ballSize.width - SHADOWOFFSET) / circleSize.width, (ballSize.height - SHADOWOFFSET) / circleSize.height);
    [[NSColor colorWithCalibratedWhite:0.875 alpha:1.0] set];
    PSarc (RADIUS-SHADOWOFFSET, RADIUS-SHADOWOFFSET, RADIUS-SHADOWOFFSET, 0.0, 360.0);
    PSfill ();

    // And the lighter & darker spots on the ball...
    [[NSColor whiteColor] set];
    PSarc (RADIUS-SHADOWOFFSET, RADIUS-SHADOWOFFSET, RADIUS-SHADOWOFFSET-1.0, 100.0, 170.0);
    PSstroke ();
    [[NSColor colorWithCalibratedWhite:0.666 alpha:1.0] set];
    PSarc (RADIUS-SHADOWOFFSET, RADIUS-SHADOWOFFSET, RADIUS-SHADOWOFFSET-1.0, 280.0, 350.0);
    PSstroke ();

    PSgrestore();
 }

 // Function to draw a shadow under the given rectangle.

- (void)drawPaddle:imageRep;		
 {
    NSRect pieceRect = {{0.0, SHADOWOFFSET}, {(paddleSize.width-SHADOWOFFSET)-1, (paddleSize.height-SHADOWOFFSET)-1}};
    drawRectangularShadowUnder (pieceRect, SHADOWOFFSET);
    draw3DBox(pieceRect, NO);
 }

- (void)drawToughTile:imageRep;		
 {
    NSRect pieceRect = {{0.0, SHADOWOFFSET}, {(tileSize.width-SHADOWOFFSET)-1, (tileSize.height-SHADOWOFFSET)-1}};
    drawRectangularShadowUnder (pieceRect, SHADOWOFFSET);
    draw3DBox(pieceRect, NO);
    pieceRect = NSInsetRect(pieceRect , 3.0 , 3.0);
    draw3DBox(pieceRect, YES);
 }

- (void)drawNormalTile:imageRep;	
 {
    NSRect pieceRect = {{0.0, SHADOWOFFSET}, {(tileSize.width-SHADOWOFFSET)-1, (tileSize.height-SHADOWOFFSET)-1}};
    drawRectangularShadowUnder (pieceRect, SHADOWOFFSET);
    draw3DBox(pieceRect, NO);
 }

- (void)drawDefaultBackground:imageRep;	
 {
 #define NOTFOUND ((id)-1)
    static NSColorList *colorList = nil;	// Static because it's shared
    NSSize boxSize = {gameSize.width / NUMXBOXES, gameSize.height / NUMYBOXES};
    int xCnt, yCnt;
    NSColor * color;

    // The first time we're here, we load and cache the color list. If we don't find
    // it, we remember that fact so that we don't go through the search again.
    if (colorList == nil) {
	NSString *colorListPath;
	if (colorListPath = [[NSBundle mainBundle] pathForResource:@"BreakApp" ofType:@"clr"]) {
	    colorList = [[NSColorList allocWithZone:NSDefaultMallocZone()] initWithName:@"" fromFile:colorListPath];
	}
	if (!colorList) {
	    NSLog(@"Can't find color list for backgrounds.");
	    colorList = NOTFOUND;
	}
    }

    // Now get the color. If the color list wasn't found, we use some default random color.
    // Note that because colors in different color spaces might look different on
    // different devices (although they might look identical on screen), it's
    // important to always use colors from the same color space when creating
    // a wash. Below we assure that our colors always start off in HSB color space.

    if (colorList != NOTFOUND) {
	color = [colorList colorWithKey:[[colorList allKeys] objectAtIndex:(level % [[colorList allKeys] count])]];
	color = [NSColor colorWithCalibratedHue:[[color colorUsingColorSpaceName:NSCalibratedRGBColorSpace] hueComponent] saturation:[[color colorUsingColorSpaceName:NSCalibratedRGBColorSpace] saturationComponent] brightness:1.0 alpha:1.0];
    } else {
	color = [NSColor colorWithCalibratedHue:(level % 8) / 7.0 saturation:0.8 brightness:1.0 alpha:1.0];
    }

    for (yCnt = 0; yCnt < NUMYBOXES; yCnt++) {
	for (xCnt = 0; xCnt < NUMXBOXES; xCnt++) {
	    float h, s, b, a;
            // Determine brightness (each box has a different brightness); we also use the color from the previous time
	    [[color colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getHue:&h saturation:&s brightness:NULL alpha:&a];
	    b = 0.4 + (yCnt + (4 - xCnt)) * (0.2 / (NUMYBOXES + NUMXBOXES));
	    // The bottom triangle
	    [[NSColor colorWithCalibratedHue:h saturation:s brightness:b alpha:a] set];
	    PSmoveto (xCnt * boxSize.width + boxSize.width / 2.0, yCnt * boxSize.height + boxSize.height / 2.0);
	    PSrlineto (-boxSize.width / 2.0, -boxSize.height / 2.0);
	    PSrlineto (boxSize.width, 0);
	    PSfill ();
	    // The right triangle
            [color = [NSColor colorWithCalibratedHue:h saturation:s brightness:b * 1.2 alpha:a] set];
	    PSmoveto (xCnt * boxSize.width + boxSize.width / 2.0, yCnt * boxSize.height + boxSize.height / 2.0);
	    PSrlineto (boxSize.width / 2.0, boxSize.height / 2.0);
	    PSrlineto (0, -boxSize.height);
	    PSfill ();
	    // The left triangle
            [color = [NSColor colorWithCalibratedHue:h saturation:s brightness:b * 1.2 * 1.2 alpha:a] set];
	    PSmoveto (xCnt * boxSize.width + boxSize.width / 2.0, yCnt * boxSize.height + boxSize.height / 2.0);
	    PSrlineto (-boxSize.width / 2.0, boxSize.height / 2.0);
	    PSrlineto (0, -boxSize.height);
	    PSfill ();
	    // The right triangle
            [color = [NSColor colorWithCalibratedHue:h saturation:s brightness:b * 1.2 * 1.2 * 1.2 alpha:a] set];
	    PSmoveto (xCnt * boxSize.width + boxSize.width / 2.0, yCnt * boxSize.height + boxSize.height / 2.0);
	    PSrlineto (boxSize.width / 2.0, boxSize.height / 2.0);
	    PSrlineto (-boxSize.width, 0.0);
	    PSfill ();
	}
    }
 }

 // The following methods show or erase the ball and the paddle from the field.

- (void)drawBackground:(NSRect)rect;	
 {
    rect.origin.x = floor(NSMinX(rect));
    rect.origin.y = floor(NSMinY(rect));
    if ([[NSDPSContext currentContext] isDrawingToScreen]) {
	[[NSColor whiteColor] set];
        NSRectFill (rect);
    }
    [backGround compositeToPoint:rect.origin fromRect:rect operation:NSCompositeSourceOver];
 }

 // drawRect:, a method every decent View should have, redraws the game
 // in its current state. This allows us to print the game very easily as well.

//
- (void)showBall;		
 {
    NSRect tmpRect = {{floor(ballX), floor(ballY)}, {ballSize.width, ballSize.height}};
    [ball compositeToPoint:tmpRect.origin operation:NSCompositeSourceOver];
 }

- (void)showPaddle;		
 {
    NSRect tmpRect = {{floor(paddleX), floor(paddleY)}, {paddleSize.width, paddleSize.height}};
    [paddle compositeToPoint:tmpRect.origin operation:NSCompositeSourceOver];
 }

- (void)eraseBall;		
 {
    NSRect rect = {{ballX, ballY}, {ballSize.width, ballSize.height}};
    [self drawBackground:rect];
 }

- (void)erasePaddle;	
 {
    NSRect rect = {{paddleX, paddleY}, paddleSize};
    [self drawBackground:rect];
 }

 // drawBackground: just draws the specified piece of the background by
 // compositing from the background image.

//
- (void)setSoundStateFrom:sender;			
 {
    if ([sender state]) {
	[SoundEffect setSoundEnabled:YES];
	if (!(soundEnabled = [SoundEffect soundEnabled])) {
	    [sender setState:NO];	// Silently fail
	}
    } else {
	soundEnabled = NO;
    }
 }

 // If user tries to enable sound once the game is launched, and it
 // fails, then we do tell him/her about it.

- (void)setSoundMode:sender;				
 {
    BOOL desiredState = [sender state];
    [self setSoundStateFrom:sender];
    if (desiredState && !soundEnabled) {
	NSRunAlertPanel(NSLocalizedString(@"No Sound", "Title of alert indicating sounds aren't available"), NSLocalizedString(@"Can't play sounds.", "Contents of alert panel"), NSLocalizedString(@"Bummer", "Acceptance that sounds can't be played"), nil, nil);
    }
 }

- (void)playSound:sound atXLoc:(float)xLoc;	
 {
    if (soundEnabled) {
	[sound play:1.0 pan:restrictValue((xLoc / gameSize.width - 0.5) * 2.0, 1.0)];
    }
 }

 // Alters the given velocity vector so that it is 
 // rotated by the indicated amount. We restrict both the resulting x and v
 // velocity values to the maximum of their max possible values...

@end