#import "SoundEffect.h"

 #import "BreakView.h"

@implementation	BreakView	(Engine)	// -----------------
// ---------------------- gotoFirstLevel: sets everything up for a new game.
- (void)gotoFirstLevel:sender;	
 {
    score = 0;
    level = 0;
    lives = LIVES;
    [self gotoNextLevel:sender];
 }

 // gotoNextLevel: sets everything up for the next level of the game; the level
 // count is incremented and the pieces are set up on the field. The ball and
 // the paddle are also brought to the starting locations.
 //
 // This routine can of course be made infinitely more complicated in
 // determining where the tiles go. Left as an exercise to the reader. 8-)

- (void)gotoNextLevel:sender;	
 {
    int xcnt, ycnt, yFrom, yTo, xFrom, xTo;
    
    // We are at the next level... Stop the game and increment the level.
    
		[self stop:sender];
		
		level++;
    
    // Now place the tiles. Here's where we could do some fancy tile layout,
    // depending on the game level. yFrom, yTo, xFrom, and xTo define the "box"
    // in which we will lay the tiles out. These values are inclusive.
    
		switch (level % 6) {
			case 0: yTo = NUMTILESY-2; break;
			case 4: yTo = NUMTILESY-4; break;
			case 5: yTo = 2 * (NUMTILESY / 3); break;
			default: yTo = 3 * (NUMTILESY / 4); break;
		}
		
		xFrom = 0; xTo = NUMTILESX-1; yFrom = yTo - 3;
		
		switch (level % 10) {
			case 1: yFrom++; break;   
			case 2: yFrom--; xFrom++; xTo--; break;
			case 4: xFrom += 2; xTo -= 2; break;
			case 6: yFrom = MIN(yFrom, NUMTILESY / 4); xFrom++; xTo--; break;
			case 7: xTo -= 3; break;
			case 8: yFrom -= 2; xFrom += 2; xTo -= 2; break;
			case 9: yFrom = MIN(yFrom, NUMTILESY / 4);
					yTo = MAX(yTo, yFrom+4);
					break;
			case 0: yFrom = MIN(yFrom, NUMTILESY / 5);
					xFrom += (NUMTILESX / 2);
					break;
			default: break;
		}    
    
    // The area in the playing field where we place tiles is at least 3 tiles 
    // high and at least NUMTILESX-4 tiles wide.
    
		// Empty out the whole playing field.
		for (xcnt = 0; xcnt < NUMTILESX; xcnt++) {
			for (ycnt = 0; ycnt < NUMTILESY; ycnt++) {
				tiles[xcnt][ycnt] = NOTILE;
			}
		}
	
		// Fill up the tile area with wimpy tiles
		for (ycnt = yFrom; ycnt <= yTo; ycnt++) {
			for (xcnt = xFrom; xcnt <= xTo; xcnt++) {
				tiles[xcnt][ycnt] = 0;
			}
		}

    // Erase or change some of the tiles, depending on the level.
    // Assumption is that we have at least 3 rows of tiles, yFrom..yTo.
    
		switch (level % 7) {
			case 2: // clear two rows in the middle	      
					for (xcnt = xFrom; xcnt <= xTo; xcnt++) {
						tiles[xcnt][yFrom+1] = tiles[xcnt][yTo-1] = NOTILE;
					}
					break;
			case 3: // randomly clear out some tiles
					for (xcnt = 0; xcnt < 5; xcnt++) {
						tiles[xFrom+RANDINT(xTo-xFrom)][yFrom+1+RANDINT(yTo-yFrom-2)] = NOTILE;
					}
					break;
			case 4: // clear middle columns
					for (xcnt = xFrom +  2; xcnt <= xTo - 2; xcnt++) {
						for (ycnt = yFrom; ycnt <= yTo; ycnt++) {
							tiles[xcnt][ycnt] = NOTILE;
						}
					}
					break;
			case 6: // clear out the insides
					for (ycnt = yFrom+1; ycnt < yTo; ycnt++) {
						for (xcnt = xFrom+1; xcnt < xTo; xcnt++) {
							tiles[xcnt][ycnt] = NOTILE;
						}
					}
					break;
			default:
					break;    
		}
    
    // Drop in some tough tiles in all rows except the first one
    for (xcnt = 0; xcnt < 5; xcnt++) {    	
		tiles[xFrom+RANDINT(xTo-xFrom)][yFrom+1+RANDINT(yTo-yFrom-1)] = 1;
    }
    
    // Compute the number of tiles we actually ended up putting down...
    numTilesLeft = 0;
    for (ycnt = yFrom; ycnt <= yTo; ycnt++) {
		for (xcnt = xFrom; xcnt <= xTo; xcnt++) {
			if (tiles[xcnt][ycnt] != NOTILE) numTilesLeft++;
		}
    }

    // Of course you might think there are too many braces in the above code,
    // where probably none would've sufficed. Too many braces never hurt, & it
    // will save you from some bozo bug some day. So use them! They're cheap!
    
    [self resetBallAndPaddle];
    
    [levelView setIntValue:level];
    [scoreView setIntValue:score];
    [livesView setIntValue:lives];
    [hscoreView setIntValue:highScore];
    [statusView setStringValue:NSLocalizedString(@"Game Ready", "Message indicating the game is ready to run")];
    
    killerBall 	= ((level % 12) == 6);	// Every 12 turns (starting on 6th), the ball loses its ability to bounce off tiles
    niceBall 	= (level % 5 == 0);		// Every 5 turns, make the ball bounce towards the paddle

    // For backgrounds from files, this is a bit wasteful...

    [backGround recache];
    
    [self setNeedsDisplay:YES];			// Display the new arrangement
    
    if (demoMode) {
		[self go:sender];	// If in demo mode, start rolling
    }
 }      

 // setDemoMode: allows the user to put the game in a demo mode.
 // In the demo mode, the paddle constantly follows the ball.

- (void)setDemoMode:sender;		
 {
    if (demoMode = ([sender state] == 0 ? NO : YES)) {
		[self go:sender];
    } else {
		[self stop:sender];
    }
 }

 // This method should be called when a new level or game is started or the
 // player misses the ball. It resets the ball & paddle locations back to
 // default.

//
- (void)resetBallAndPaddle;			
 {
    paddleX = PADDLEX;
    paddleY = PADDLEY;
    ballX = BALLX;
    ballY = BALLY;

    ballXVel = 0.0;
    ballYVel = 0.0;

    // The ball shouldn't start out rotating...
    revolutionsLeft = 0;	
 }
        
 // The directBallAt: initializes the velocity vector of the ball so that
 // the ball will go from its current location to the specified destination  
 // point. The speed of the ball is determined by the current level. If ballYVel
 // is already set, then only the x velocity & y direction is changed.

- (void)directBallAt:(NSPoint)dest;	
 {
    float desiredYVel = dest.y - (ballY + ballSize.height / 2.0);
    float desiredXVel = dest.x - (ballX + ballSize.width / 2.0);

    // Transform back to original game coords (velocity values are measured
    // in these).

    desiredYVel /= (gameSize.height / GAMEHEIGHT);
    desiredXVel /= (gameSize.width / GAMEWIDTH);

    if (fabs(desiredYVel) < 1.0) {
	desiredYVel = desiredYVel < 0.0 ? -1.0 : 1.0;
    }
    if (ballYVel == 0.0) {
	// Come up with a value between 60 and 100% of MAXYV.
	ballYVel = restrictValue(((RANDINT(level * 8) + 60.0) / 100.0) * MAXYV, MAXYV);
    }
    ballYVel = fabs(ballYVel) * (desiredYVel < 0.0 ? -1.0 : 1.0);
    ballXVel = restrictValue(ballYVel * (desiredXVel / desiredYVel) ,MAXXV);
 }    

 // The stop method will pause a running game. The go method will start it up
 // again. They can be assigned to buttons or other appkit objects through IB.

- (void)go:sender;					
 {
    if (lives && !gameRunning) {
		// If the ball velocity wasn't initialized, start it rolling
		// towards the mouse location...
		if (ballXVel == 0.0 && ballYVel == 0.0) {
			NSPoint mouseLoc = [self convertPoint:[[self window] mouseLocationOutsideOfEventStream] fromView:nil];
			[self directBallAt:mouseLoc];
			ballYVel = fabs(ballYVel);
		}
		gameRunning = YES;
		timer = [[NSTimer scheduledTimerWithTimeInterval:0.03 target:self selector:@selector(step:) 
																	userInfo:self repeats:YES] retain];
		[statusView setStringValue:NSLocalizedString(@"Running", "Message indicating that the game is running")];
    }
 }

- (void)stop:(id)sender;			
 {
    if (gameRunning) {
		gameRunning = NO;
		[timer invalidate]; [timer release];;
		[statusView setStringValue:NSLocalizedString(@"Paused", "Message indicating that the game is paused")]; 
    }
 }

- (void)rotate:(float *)xVel :(float *)yVel by:(float)radians;	
 {
    float newAngle = atan2 (*yVel, *xVel) + radians;
    float velocity = hypot (*xVel, *yVel); 

    *yVel = restrictValue(velocity * sin(newAngle), MAX(MAXYV, MAXXV));
    *xVel = restrictValue(velocity * cos(newAngle), MAX(MAXYV, MAXXV));
 }

 // The step method implements one step through the main game loop.
 // The distance traveled by the ball is adjusted by the time between frames.

- (void)step:(NSTimer *)timer;		
 {
    NSPoint mouseLoc;
    float newX;
    NSTimeInterval 	timeNow = [NSDate timeIntervalSinceReferenceDate];
    unsigned int 	timeDelta = MIN(MAX((timeNow - lastFrameTime) * 1000, MINTIMEDIFFERENCE), MAXTIMEDIFFERENCE);
    lastFrameTime = timeNow;
   
    [self lockFocus];
    
    [self eraseBall];
    
    // If the ball is rotating, rotate it by the indicated amount.

    if (revolutionsLeft > 0.0) {
		float revsThisTime = revolutionSpeed * timeDelta;
		[self rotate:&ballXVel :&ballYVel by:revsThisTime];
		revolutionsLeft -= revsThisTime;
		if (revolutionsLeft <= 0.0 && (fabs(ballYVel) < MAXYV * 0.6)) {
			// Done rotating; make sure we have a good y-velocity
			ballYVel = MAXYV * 0.8 * (ballYVel < 0.0 ? -1 : 1);
			ballXVel = restrictValue(ballXVel,MAXXV);
		}
    } else if (ONEIN(1000 + (level < 8 ? (8 - level) * 250 : 0)) && (ballY > gameSize.height * 0.6)) {
		// If we're not rotating, we go into rotating mode one out of 
		// 1500 or more steps, provided that the ball is not too close to
		// the paddle at the time.
        revolutionsLeft = APPROX_PI * (2 + RANDINT(5)); // 1 to 3.5 full turns
		revolutionSpeed = MAXREVOLUTIONSPEED * (RANDINT(8) + 2.0) / 10.0;
    } 

    // Update the ball location

    ballX += ballXVel * timeDelta * gameSize.width / GAMEWIDTH; 
    ballY += ballYVel * timeDelta * gameSize.height / GAMEHEIGHT;


    if (gameRunning) {
	
		if (ballX < 0.0) { // Hit on the left wall
			ballX = 0.0;
			ballXVel = -ballXVel; 
			[self playSound:wallSound atXLoc:ballX];
		} else if (ballX > gameSize.width - ballSize.width) { // Right wall
			ballX = gameSize.width - ballSize.width;
			ballXVel = -ballXVel; 
			[self playSound:wallSound atXLoc:ballX];
		}
	
		if (ballY > gameSize.height - ballSize.height) { // Top wall
			ballY = gameSize.height - ballSize.height;
			ballYVel = -ballYVel;
			if (niceBall && !ONEIN(5) && !demoMode) {
				NSPoint mid = {paddleX + paddleSize.width / 2.0, paddleY};
				[self directBallAt:mid];
			} else if (ONEIN(10)) {
				ballXVel = MAXXV-(RANDINT((int)(MAXXV*20))/10.0);
			}
			[self playSound:wallSound atXLoc:ballX];
		}
	
		// Now checking for collisions with tiles... 
	
		{
			int y1 = (int)(floor(ballY / (tileSize.height + INTERTILE)));
			int x1 = (int)(floor((ballX - leftMargin) / (tileSize.width + INTERTILE)));
			int y2 = (int)(floor((ballY + ballSize.height) / (tileSize.height + INTERTILE)));
			int x2 = (int)(floor((ballX + ballSize.width - leftMargin) / (tileSize.width + INTERTILE)));
		
			if ([self hitTileAt:x1 :y1] | [self hitTileAt:x2 :y1] | [self hitTileAt:x1 :y2] | [self hitTileAt:x2 :y2]) {
				[self playSound:tileSound atXLoc:ballX];
				if (!killerBall) {
					ballYVel = -ballYVel;
				}
				[scoreView setIntValue:score];
				[[self window] flushWindow];
			}
		}
    }

    // Get the mouse location and convert from window to the view coords.
    // If in demo, mode, make the paddle track the ball. Endless fun.

    if (demoMode) {
		mouseLoc.x = ballX + ballSize.width / 2.0;
    } else {
		mouseLoc = [[self window] mouseLocationOutsideOfEventStream];
		mouseLoc = [self convertPoint:mouseLoc fromView:nil];
    }

    newX = MAX(MIN(mouseLoc.x - paddleSize.width/2,  gameSize.width - paddleSize.width), 0);

    if (ballY >= paddleY + paddleSize.height) {

		// Ball is above the paddle; redraw it and the paddle and continue
		// We flush twice as the ball and the paddle are not too close together
	
		[self showBall];
		[[self window] flushWindow];
		[self erasePaddle];
		paddleX = newX;
		[self showPaddle];
		[[self window] flushWindow];

    } else if (ballY + ballSize.height > 0) {
		
		// Ball is past the paddle but not totally gone...
	
		[self erasePaddle];
		paddleX = newX;
	
		// Check to see if the user managed to catch the ball after all
	
		if ((ballY > paddleY - ballSize.height / 2.0) &&
			(ballX <= paddleX + paddleSize.width) &&
			(ballX + ballSize.width > paddleX)) {
			[self paddleHit];
		}
	
		// The ball and the paddle are close, so one flushWindow is fine.
	
		[self showBall];
		[self showPaddle];
		[[self window] flushWindow];

    } else {
	
		// Too late; the ball is out of sight...
	
		[self erasePaddle];
		[self stop:self];
		[self playSound:missSound atXLoc:0.0];
	
		if (--lives == 0) {
			if (score > highScore) [self setHighScore:score];
			[statusView setStringValue:NSLocalizedString(@"Game Over", "Message indicating that the game is over...")];
		} else {
			[self resetBallAndPaddle]; 
			[self showBall];
			[self showPaddle]; 
			[statusView setStringValue:NSLocalizedString(@"Game Ready", "Message indicating the game is ready to run")];
		}
		[[self window] flushWindow];
	
		[livesView setIntValue:lives];
		
    }

    // numTilesLeft <= 0 indicates that we've blown away every tile. But,
    // to make the game more exciting, we start decrementing numTilesLeft, 
    // by one everytime through this loop, until it reaches the value 
    // STOPGAMEAT. This makes the ball move a bit more after all the tiles 
    // are gone. But, if gameRunning is NO, then it means we probably just
    // missed the ball, in which case we should go ahead and jump to the 
    // next level.
    
    if ((numTilesLeft <= 0) && ((lives && !gameRunning) || (--numTilesLeft == STOPGAMEAT))) {
		[self incrementGameScore:LEVELBONUS];
		[self gotoNextLevel:self];
    }

    PSWait ();	// Synchronize postscript for smoother animation

    [self unlockFocus];
 }

//
- (BOOL) hitTileAt:(int)x :(int)y;				
 {
    NSRect rect = {{floor(leftMargin + (tileSize.width + INTERTILE) * x), 
		    floor((tileSize.height + INTERTILE) * y)},
		   {tileSize.width, tileSize.height}};

    if (x < NUMTILESX && y < NUMTILESY && x >= 0 && y >= 0 && 
	(tiles[x][y] != NOTILE)) {
	[self incrementGameScore:tileScores[tiles[x][y]]];
	ballYVel = restrictValue(ballYVel * tileAccs[tiles[x][y]], MAXYV);
	[self drawBackground:rect];
	tiles[x][y] = NOTILE;
	numTilesLeft--;
	return YES;
    } else {
	return NO;
    }
 }


 // The paddleHit method is called whenever the ball hits the paddle.
 // This method bounces the ball back at an angle depending on what part of
 //  the paddle was hit.

- paddleHit;									
 {
    float whereHit = ((ballX + RADIUS) - paddleX) / paddleSize.width;

    ballYVel = -ballYVel;
    ballY = paddleSize.height;

    [self playSound:paddleSound atXLoc:paddleX];

    // Alter the x-velocity and make sure it is in the valid range.
    // If the ball hits the edges of the paddle, bounce it back at some angle.
    
    if (whereHit < 0.1) {
	ballXVel = - MAXXV;
    } else if (whereHit > 0.9) {
	ballXVel = MAXXV;
    } else {
	// Now whereHit is in the range 0.1 .. 0.9, with 0.5 indicating middle
	// of the paddle.  Convert to a number in the range 0.2 to 1, with 0.2
	// indicating the middle and 1 either end.
	whereHit = (fabs(whereHit - 0.5) + 0.1) * 2.0;  
	ballXVel = ((ballXVel > 0.0) ? 1.0 : -1.0) * MAXXV * whereHit;
    }

    return self;
 }

 // If upon launch we discover that there's no sound, then we fail
 // silently. Note that although the SoundEffect class has the ability
 // to enable/disable sounds, because we might have multiple
 // BreakViews each with its own sound state, we keep a local state in
 // addition to the one in SoundEffect.

 // Note that although this is an outlet method, we do not actually
 // have an outlet (instance variable) named soundStateFrom; we just
 // want to take a look at the initial value of the button and see
 // if sound needs to be turned on...

@end

