#import "SoundEffect.h"
 
 #import "BreakView.h"

@implementation	BreakView	(Overridden)	// ---------------------------
- (void)getHighScore;							
 {
    highScore = [[NSUserDefaults standardUserDefaults] integerForKey:@"HighScore"];
 }

 // setHighScore should be called when the user score for a game is above 
 // the current high score. setHighScore sets the high score and 
 // writes it out the defaults file so that it can be remembered for eternity.

- (void)setHighScore:(int)hScore;				
 {
    highScore = hScore;
    [[NSUserDefaults standardUserDefaults] setInteger:highScore forKey:@"HighScore"];
 }

- (void)incrementGameScore:(int)scoreIncrement;	
 {
    if (demoMode == NO) {
	score += scoreIncrement;
    }
 }

 // hitTileAt:: checks to see if there's a tile at tile location x, y;
 // if so, it is considered hit by the ball and cleared. hitTileTile:: also
 // updates the score and the ball velocity. hitTileAt:: returns YES if there
 // was a tile, NO otherwise.

- (int)score;						{ return score; }
- (int)level;						{ return level; }
- (int)lives;						{ return lives; }
//
// - (void)setScore:(int)number;			// No this would be cheating ...
- (void)setLevel:(int)number;			{ level = number; }
- (void)setLives:(int)number;			{ lives = number; }
//
- (id)initWithFrame:(NSRect)frm;
 {
    [super initWithFrame:frm];
    
    [self allocateGState];	// For faster lock/unlockFocus
    
    [(ball = [[NSImage allocWithZone:[self zone]] init]) setScalesWhenResized:NO];
    [ball addRepresentation:[[[NSCustomImageRep alloc] initWithDrawSelector:@selector(drawBall:) delegate:self] autorelease]];

    [(paddle = [[NSImage allocWithZone:[self zone]] init]) setScalesWhenResized:NO];
    [paddle addRepresentation:[[[NSCustomImageRep alloc] initWithDrawSelector:@selector(drawPaddle:) delegate:self] autorelease]];

    [(tile[0] = [[NSImage allocWithZone:[self zone]] init]) setScalesWhenResized:NO];
    [(tile[1] = [[NSImage allocWithZone:[self zone]] init]) setScalesWhenResized:NO];
    [tile[0] addRepresentation:[[[NSCustomImageRep alloc] initWithDrawSelector:@selector(drawNormalTile:) delegate:self] autorelease]];
    [tile[1] addRepresentation:[[[NSCustomImageRep alloc] initWithDrawSelector:@selector(drawToughTile:) delegate:self] autorelease]];

    wallSound = [[SoundEffect allocWithZone:[self zone]] initWithSoundResource:@"Wall"];
    tileSound = [[SoundEffect allocWithZone:[self zone]] initWithSoundResource:@"Tile"];
    missSound = [[SoundEffect allocWithZone:[self zone]] initWithSoundResource:@"Miss"];
    paddleSound = [[SoundEffect allocWithZone:[self zone]] initWithSoundResource:@"Paddle"];

    [self setBackgroundFile:[[NSUserDefaults standardUserDefaults] stringForKey:@"BackGround"] andRemember:NO];
	    
    [self resizePieces];
    
    [self getHighScore];
    
    demoMode = NO;
    
    INITRAND;
    
    return self;
 }

 // free simply gets rid of everything we created for BreakView, including
 // the instance of BreakView itself. This is how nice objects clean up.

- (void)dealloc;				
 {
    int cnt;

    if (gameRunning) {
	[timer invalidate];
        [timer release];
    }
    for (cnt = 0; cnt < NUMTILETYPES; cnt++) {
	[tile[cnt] release];
    }

    [ball release];    
    [paddle release];
    [backGround release];
    [wallSound release];
    [tileSound release];
    [missSound release];
    [paddleSound release];

    [super dealloc];
 }

 // resizePieces calculates the new sizes of all the pieces after the game is
 // started or the playing field (the BreakView) is resized.

//
- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent;		 { return YES; }

 // This methods allows changing the file used to paint the background of the
 // playing field. Set fileName to NULL to revert to the default. Set
 // remember to YES if you wish the write the value out in the defaults.

- (void)mouseDown:(NSEvent *)event;		
 {
    if (gameRunning) {
		[self stop:self]; 
    } else if (lives) {
		[self go:self];   
    }
 }

 // The following few methods draw the pieces.

- (void)setFrameSize:(NSSize)_newSize;	
 {
    NSSize oldSize = [self bounds].size;
    
    [super setFrameSize:_newSize];
    
    ballX = (ballX * _newSize.width / oldSize.width);
    ballY = (ballY * _newSize.height / oldSize.height);
    paddleX = (paddleX * _newSize.width / oldSize.width);
    paddleY = (paddleY * _newSize.height / oldSize.height);
    
    [self resizePieces];
    
    [self setNeedsDisplay:YES];
 }

 // A mousedown effectively allows pausing and unpausing the game by
 // alternately calling one of the above two functions (stop/go).

- (void)drawRect:(NSRect)rect;			
 {
    int xcnt, ycnt;

    [self drawBackground:rect];

    for (xcnt = 0; xcnt < NUMTILESX; xcnt++) { 
	for (ycnt = 0; ycnt < NUMTILESY; ycnt++) {
	    if (tiles[xcnt][ycnt] != NOTILE) {
		NSPoint tileLoc = {floor(leftMargin + (tileSize.width + INTERTILE) * xcnt), floor((tileSize.height + INTERTILE) * ycnt)};
		[tile[tiles[xcnt][ycnt]] compositeToPoint:tileLoc operation:NSCompositeSourceOver];
	    }
	}
    }

    if (lives) {
	[self showBall];
	[self showPaddle];
    }
 }

 // incrementGameScore: adds the value of the argument to the score if the game
 // is not in demo mode.

//
- (void)resizePieces;					
 {
    int cnt;
    float xRatio = gameSize.width / GAMEWIDTH;
    float yRatio = gameSize.height / GAMEHEIGHT;

    [backGround setSize:gameSize];

    tileSize.width = floor(xRatio * TILEWIDTH);
    tileSize.height = floor(yRatio * TILEHEIGHT);
    for (cnt = 0; cnt < NUMTILETYPES; cnt++) {
	[tile[cnt] setSize:tileSize];
    }
    leftMargin = floor((gameSize.width - (tileSize.width + INTERTILE) * NUMTILESX) / 2.0 + 1.0);

    paddleSize.width = floor(xRatio * PADDLEWIDTH);
    paddleSize.height = floor(yRatio * PADDLEHEIGHT);
    [paddle setSize:paddleSize];

    ballSize.width = floor(xRatio * BALLWIDTH);
    ballSize.height = floor(yRatio * BALLHEIGHT);
    [ball setSize:ballSize];
 }

 // The following allows BreakView to grab the mousedown event that activates
 // the window. By default, the View's acceptsFirstMouse returns NO.


@end
