//ADHeader// #import <Foundation/Foundation.h>
//ADHeader// #import <AppKit/AppKit.h>
//ADHeader// 
//ADHeader// // Maximum number of tiles in the playing area...
//ADHeader// 
//ADHeader// #define NUMTILESX 8      
//ADHeader// #define NUMTILESY 20
//ADHeader// 
//ADHeader// #define NUMTILETYPES 2   // Number of different tile types.
//ADHeader// 
//ADHeader// #define TILEHEIGHT	18.0  // Tile size, in pixels
//ADHeader// #define TILEWIDTH	28.0				
//ADHeader// #define INTERTILE	2.0
//ADHeader// 
//ADHeader// #define GAMEWIDTH  ((TILEWIDTH+INTERTILE) * NUMTILESX)  // Size of the field.
//ADHeader// #define GAMEHEIGHT ((TILEHEIGHT+INTERTILE) * NUMTILESY)
//ADHeader// 
//ADHeader// /*
//ADHeader//  BreakView.m, view to implement the "BreakApp" game.
//ADHeader//  Author: Ali Ozer
//ADHeader//  Written for 0.8 October 88. 
//ADHeader//  Modified for 0.9 March 89.
//ADHeader//  Modified for 1.0 July 89.
//ADHeader//  Removed use of Bitmap and threw away some classes May 90.
//ADHeader//  Final 2.0 fixes/enhancements Sept 90.
//ADHeader//  3.0 update March 92.
//ADHeader//  Sound-related changes for 3.1 March 92.
//ADHeader//  Modified for 4.0 Jan 96
//ADHeader//  Modified for Rhapsody June 97
//ADHeader// 
//ADHeader//  BreakView implements an interactive custom view that allows the user
//ADHeader//  to play "BreakApp," a game similar to a popular arcade classic.
//ADHeader// 
//ADHeader//  BreakView's main control methods are based on the target-action
//ADHeader//  paradigm; thus you can include BreakView in an Interface-Builder based
//ADHeader//  application. Please refer to BreakView.h for a list of "public" methods
//ADHeader//  that you should provide links to in Interface Builder.
//ADHeader//  
//ADHeader//  Copyright (c) 1988-1997 by Apple Computer, Inc., all rights reserved.
//ADHeader// 
//ADHeader//  You may incorporate this sample code into your applications without
//ADHeader//  restriction, though the sample code has been provided "AS IS" and the
//ADHeader//  responsibility for its operation is 100% yours.  However, what you are
//ADHeader//  not permitted to do is to redistribute the source as "DSC Sample Code"
//ADHeader//  after having made changes. If you're going to re-distribute the source,
//ADHeader//  we require that you make it clear in the source that the code was
//ADHeader//  descended from Apple Sample Code, but that you've made changes.
//ADHeader// */
//ADHeader// 
//ADHeader// 
//ADSeparator//
#import "BreakView.h"

@implementation	BreakView  :  NSView	// #####################################
{
   int 			numTilesLeft, score, level, lives, highScore;
   BOOL 		gameRunning, demoMode, soundEnabled;

   NSTimer *		timer;
   NSTimeInterval 	lastFrameTime;

   // The various pieces that know how to draw themselves on the field.
   id 		tile[NUMTILETYPES];

   // The actual array of tiles on the field. tiles[x][y] is the id of the
   // tile that is at location x * TILEWIDTH, y * TILEHEIGHT.

   short	 tiles[NUMTILESX][NUMTILESY];
   id 		ball, paddle, backGround;

   // The following six are outlets set when the nib file is being read in.
   
   // Views that display various info (these can be nil, in which case the
   // corresponding info will just go to the bit bucket). 
   
   id 	scoreView;
   id 	livesView;
   id 	levelView;
   id 	hscoreView;
   id 	statusView;

   id	 wallSound;
   id	 tileSound;
   id	 paddleSound;
   id	 missSound;
   
   // Other ball & paddle params
   float 		ballX, ballY, paddleX, paddleY, ballXVel, ballYVel, leftMargin;

   NSSize	 ballSize, paddleSize, tileSize;

   // These variables store the of the ball. Killer ball goes through tiles
   // without bouncing; nice ball bounces from the top wall towards the paddle. 
   BOOL 	killerBall, niceBall;

   // Number of revolutions left if the ball is rotating. If zero or
   // less, than the ball is not rotating...
   float 		revolutionsLeft;
   float 		revolutionSpeed; // Radians per millisecond if rotating
}
@end

// Convert x-location to left/right pan for playing sounds
float restrictValue(float val, float max) {
    if (val > max) return max;
    else if (val < -max) return -max;
    else return val;
 }


//ADSeparator//
//ADHeader// #import "BreakView+Defines.h"
//ADHeader// #import "BreakView+Overridden.h"
//ADHeader// #import "BreakView+Engine.h"
//ADHeader// #import "BreakView+DrawingSound.h"
//ADHeader// 