#import "CalcController.h"
#import "Buttons.h"

#import "ActiveDeveloperLib.h"

@implementation	CalcController	(Methods)
- (void)awakeFromNib				
 {
    [self clear: self];
    polish = NO;
    [[displayView window] makeKeyAndOrderFront:nil];
 }

- (void)dealloc						
 // when self is released, it is necessary to release all of the variables we've retained
 {
    [displayedNumber release];
    [enteredNumber release];

    [upperStackedNumber release];
    [middleStackedNumber release];
    [lowerStackedNumber release];
 }

- (NSString *)description;			
 {
 id string = [NSString stringWithFormat: @"%@  Entering %d  DecimalPoint %d  DisplayedNr %@  enteredNr %@  lastOp %d", 
 						[super description], enteringNumber, usedDecimalPoint, displayedNumber, enteredNumber, lastOperation];

	return string;
 }
- (void)applicationDidFinishLaunching:(NSNotification *)notification;						
 {
 //NSApplication *theApplication = [notification object];

	if (ActiveDeveloper_connectToIDE())		NSLog(@"  Application connected to ActiveDeveloper IDE ... ");
	else							NSLog(@"  Application didn't connect to ActiveDeveloper IDE ...");
 }	

- (BOOL)applicationShouldTerminate:(id)sender;												
 {
 	ActiveDeveloper_applicationShouldTerminate(sender);
	return YES;
 }
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication		
 {
    return YES;
 }


//
- (NSDecimalNumber *)displayedNumber;
    // This way of detecting NANs is a HACK!  Unfortunately, NaN is not ordered, so
    // if you ask any number if it's equal to NaN, it'll say yes.  So, instead we'll
    // ask if it's equal to an impossible combination of numbers -- zero and one.
    // The only 'number' equal to both is NaN
 {
    if (([displayedNumber compare: [NSDecimalNumber zero]] == NSOrderedSame) &&
     	    ([displayedNumber compare: [NSDecimalNumber one]] == NSOrderedSame))
      {
        displayedNumber = [NSDecimalNumber notANumber];
      }

    return displayedNumber;	
 }
- (NSDecimalNumber *)enteredNumber;
    // This way of detecting NANs is a HACK!  Unfortunately, NaN is not ordered, so
    // if you ask any number if it's equal to NaN, it'll say yes.  So, instead we'll
    // ask if it's equal to an impossible combination of numbers -- zero and one.
    // The only 'number' equal to both is NaN
 {
    if (([enteredNumber compare: [NSDecimalNumber zero]] == NSOrderedSame) &&
     	    ([enteredNumber compare: [NSDecimalNumber one]] == NSOrderedSame))
      {
        enteredNumber = [NSDecimalNumber notANumber];
      }

    return enteredNumber;	
 }
- (NSDecimalNumber *)upperStackedNumber;		{ return upperStackedNumber; }
- (NSDecimalNumber *)middleStackedNumber;		{ return middleStackedNumber; }
- (NSDecimalNumber *)lowerStackedNumber;		{ return lowerStackedNumber; }
//
- (void)displayedNumber:(NSDecimalNumber *)number;
 {
    [displayedNumber autorelease];
    displayedNumber = [number retain];
 }
- (void)enteredNumber:(NSDecimalNumber *)number;
 {
    [enteredNumber autorelease];
    enteredNumber = [number retain];
 }
- (void)upperStackedNumber:(NSDecimalNumber *)number;
 {
    [upperStackedNumber autorelease];
    upperStackedNumber = [number retain];
 }
- (void)middleStackedNumber:(NSDecimalNumber *)number;
 {
    [middleStackedNumber autorelease];
    middleStackedNumber = [number retain];
 }
- (void)lowerStackedNumber:(NSDecimalNumber *)number;
 {
    [lowerStackedNumber autorelease];
    lowerStackedNumber = [number retain];
 }
//
- (void)polishPush:(NSDecimalNumber *)number
 {
	[self upperStackedNumber: [self middleStackedNumber]];
	[self middleStackedNumber: [self lowerStackedNumber]];
	[self lowerStackedNumber:number];

	NSLog(@"PolishPush: %@", number);
 }
- (NSDecimalNumber *)polishPop;
 {
 id result = [self lowerStackedNumber];

	[self lowerStackedNumber: [self middleStackedNumber]];
	[self middleStackedNumber: [self upperStackedNumber]];
	[self upperStackedNumber: [NSDecimalNumber zero]];

	NSLog(@"PolishPop: %@", result);
	return result;
 }
- (void)clear:(id)sender			
 {
    [self displayedNumber: [NSDecimalNumber zero]];
    [self enteredNumber: [NSDecimalNumber zero]];

    [self upperStackedNumber: [NSDecimalNumber zero]];
    [self middleStackedNumber: [NSDecimalNumber zero]];
    [self lowerStackedNumber: [NSDecimalNumber zero]];

    [displayView setDoubleValue: 0];
    [self endEditingNumber];
    usedDecimalPoint = NO;

    lastOperation = kNoOperator;
 }

- (void)polish:(id)sender;
 {
	if ([sender state] == 1)	polish = YES;
	else				polish = NO;

	NSLog(@"Polish state %d", polish);
	[self clear: self];
 }
- (void)equals:(id)sender			
 {
    if (enteringNumber)
      {
        [self endEditingNumber];

        //[self performLastOperation];
        [sender performOperation:lastOperation with:self];
        lastOperation = kNoOperator;

        [displayView setStringValue: [displayedNumber stringValue]];
      }
 }

- (void)operation:(id)sender		
 {
    if (enteringNumber)    [self endEditingNumber];

    if (polish) {
      	[sender performPolishOperation:[sender tag] with:self];
 	[displayView setStringValue: [displayedNumber stringValue]];

	NSLog(@"upper: %@,   middle: %@,   lower: %@,   displayed: %@", 
			upperStackedNumber, middleStackedNumber, lowerStackedNumber, [self displayedNumber]);

    } else   if ([sender tag] < kSpaceOperation) {

	//[self performLastOperation];
	[sender performOperation:lastOperation with:self];

	[displayView setStringValue: [displayedNumber stringValue]];

	// no matter what, set the new operation in case the user changes his/her mind
	lastOperation = [sender tag];
    }

 }

//
- (void)insertDigit:(id)sender		
 {
    NSString* oldValue;

    if (!enteringNumber)  [self beginEditingNumber];

    oldValue = [displayView stringValue];
    [displayView setStringValue: [oldValue stringByAppendingString: [sender title]]];
 }

- (void)decimalPoint:(id)sender		
 {
    NSString* oldValue;

    if (!usedDecimalPoint) {
        usedDecimalPoint = YES;

        if (!enteringNumber)  {
            [self beginEditingNumber];
            [displayView setStringValue: @"0."];
		} else  {
            oldValue = [displayView stringValue];
            [displayView setStringValue: [oldValue stringByAppendingString: @"."]];
		}
	}
 }

//
- (void)beginEditingNumber			
 {
    enteringNumber = YES;
    usedDecimalPoint = NO;
    [displayView setStringValue: @""];

    if (polish)	[self polishPush: [displayedNumber copy]];
   else		[self enteredNumber: [displayedNumber copy]]; 
 }

- (void)endEditingNumber			
 {
    enteringNumber = NO;
    usedDecimalPoint = NO;

    [self displayedNumber: [NSDecimalNumber decimalNumberWithString: [displayView stringValue]]];
 }

//
- (void)cut:(id)sender				
 {
    [self copy: self];
    [self clear: self];
 }

- (void)copy:(id)sender				
 {
    NSPasteboard* pasteboard = [NSPasteboard generalPasteboard];
    [pasteboard declareTypes: [NSArray arrayWithObject: NSStringPboardType] owner: NULL];
    [pasteboard setString: [displayView stringValue] forType: NSStringPboardType];
 }

- (void)paste:(id)sender			
 {
    NSPasteboard* pasteboard;
    NSString* pasteValue;

    if (!enteringNumber)
      {
        [self beginEditingNumber];
      }

    pasteboard = [NSPasteboard generalPasteboard];
    pasteValue = [pasteboard stringForType: NSStringPboardType];
    [displayView setStringValue: [[displayView stringValue] stringByAppendingString: pasteValue]];
 }

@end
